# -*- coding: utf-8 -*-
"""Authenticates from the CLI"""

from oktaauthpy3 import metadata


__version__ = metadata.version
__author__ = metadata.authors[0]
__license__ = metadata.license
__copyright__ = metadata.copyright



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Program entry point"""

from __future__ import print_function

import argparse
import sys
import getpass
import logging

from oktaauthpy3 import models
from oktaauthpy3 import metadata

log = logging.getLogger('oktaauthpy3')

def configurelogging():
    log.setLevel(logging.DEBUG)
    stderrlog = logging.StreamHandler()
    stderrlog.setFormatter(logging.Formatter("%(message)s"))
    log.addHandler(stderrlog)

def main(argv):
    configurelogging()

    """Program entry point.

    :param argv: command-line arguments
    :type argv: :class:`list`
    """
    author_strings = []
    for name, email in zip(metadata.authors, metadata.emails):
        author_strings.append('Author: {0} <{1}>'.format(name, email))

    epilog = '''
{project} {version}

{authors}
URL: <{url}>
'''.format(
        project=metadata.project,
        version=metadata.version,
        authors='\n'.join(author_strings),
        url=metadata.url)

    arg_parser = argparse.ArgumentParser(
        prog=argv[0],
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=metadata.description,
        epilog=epilog)

    arg_parser.add_argument(
        '-V', '--version',
        action='version',
        version='{0} {1}'.format(metadata.project, metadata.version))

    arg_parser.add_argument(
        '-s', '--server', type=str, help='Okta server', required=True)
    arg_parser.add_argument(
        '-u', '--username', type=str, help='Username', required=True)
    arg_parser.add_argument(
        '-t', '--apptype', type=str, help='Application type', required=True)
    arg_parser.add_argument(
        '-i', '--appid', type=str, help='Application id', required=True)

    config = arg_parser.parse_args(args=argv[1:])

    log.info(epilog)

    log.debug("Server: %s" % config.server)
    log.debug("Username: %s" % config.username)
    log.debug("Application type: %s" % config.apptype)
    log.debug("Application ID: %s" % config.appid)

    password = getpass.getpass()
    passcode = getpass.getpass("Passcode: ")
    okta = models.OktaSamlAuth(config.server,
                               config.apptype, config.appid,
                               config.username, password, passcode)

    try:
        print(okta.auth())
        return 0
    except Exception as e:
        log.error('Error authorising with okta: %s' % e)
        return 1
    finally:
        del password
        del passcode


def entry_point():
    """Zero-argument entry point for use with setuptools/distribute."""
    raise SystemExit(main(sys.argv))


if __name__ == '__main__':
    entry_point()



# -*- coding: utf-8 -*-
"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'oktaauthpy3'
project = "Okta CLI authentication with Python3 support"
project_no_spaces = project.replace(' ', '')
version = '0.8'
description = 'Authenticates from the CLI'
authors = ['Peter Gillard-Moss']
authors_string = ', '.join(authors)
maintainers = ['Mansab Uppal']
maintainers_string = ', '.join(maintainers)
emails = ['muppal@thoughtworks.com']
license = 'Apache 2.0'
copyright = '2015 Thoughtworks Inc'
url = 'https://www.thoughtworks.com/'



# -*- coding: utf-8 -*-
from pytest import raises

# The parametrize function is generated, so this doesn't work:
#
#     from pytest.mark import parametrize
#
import pytest
parametrize = pytest.mark.parametrize

from oktaauthpy3 import metadata
from oktaauthpy3.main import main


class TestMain(object):
    @parametrize('helparg', ['-h', '--help'])
    def test_help(self, helparg, capsys):
        with raises(SystemExit) as exc_info:
            main(['progname', helparg])
        out, err = capsys.readouterr()
        # Should have printed some sort of usage message. We don't
        # need to explicitly test the content of the message.
        assert 'usage' in out
        # Should have used the program name from the argument
        # vector.
        assert 'progname' in out
        # Should exit with zero return code.
        assert exc_info.value.code == 0

    @parametrize('versionarg', ['-V', '--version'])
    def test_version(self, versionarg, capsys):
        with raises(SystemExit) as exc_info:
            main(['progname', versionarg])
        out, err = capsys.readouterr()
        # Should print out version.
        assert err == '{0} {1}\n'.format(metadata.project, metadata.version)
        # Should exit with zero return code.
        assert exc_info.value.code == 0



import requests
import base64
from urllib.parse import urlparse
from urllib.parse import urlunparse
import logging
from bs4 import BeautifulSoup
log = logging.getLogger('oktaauthpy3')

class OktaAPIAuth(object):

    def __init__(self, okta_server, username, password, passcode):
        passcode_len = 6
        self.okta_url = None
        self.username = username
        self.password = password
        self.passcode = passcode
        url_new = ('https', okta_server,
                   '', '', '','')
        self.okta_url = urlunparse(url_new)
        return

    def okta_req(self, path, data):
        url = '{base}/api/v1{path}'.format(base=self.okta_url, path=path)
        resp = requests.post(url=url, headers={'Accept': 'application/json',
                                               'Content-Type': 'application/json'}, json=data)
        return resp.json()

    def preauth(self):
        path = '/authn'
        data = {'username': self.username,
                'password': self.password}
        return self.okta_req(path, data)

    def doauth(self, fid, state_token):
        path = '/authn/factors/{fid}/verify'.format(fid=fid)
        data = {'fid': fid,
                'stateToken': state_token,
                'passCode': self.passcode}
        return self.okta_req(path, data)

    def auth(self):
        username = self.username
        password = self.password
        status = False
        rv = False
        invalid_username_or_password = username is None or username == '' or password is None or password == ''
        if invalid_username_or_password:
            log.info("Missing username or password for user: {} ({}) - Reported username may be 'None' due to this".format(username))
            return False
        else:
            if not self.passcode:
                log.info('No second factor found for username %s' % username)
            log.debug('Authenticating username %s' % username)
            try:
                rv = self.preauth()
            except Exception as s:
                log.error('Error connecting to the Okta API: %s' % s)
                return False

            if 'errorCauses' in rv:
                msg = rv['errorSummary']
                log.info('User %s pre-authentication failed: %s' % (self.username, msg))
                return False
            if 'status' in rv:
                status = rv['status']
            if status == 'SUCCESS':
                log.info('User %s authenticated without MFA' % self.username)
                return rv['sessionToken']
            if status == 'MFA_ENROLL' or status == 'MFA_ENROLL_ACTIVATE':
                log.info('User %s needs to enroll first' % self.username)
                return False
            if status == 'MFA_REQUIRED' or status == 'MFA_CHALLENGE':
                log.debug('User %s password validates, checking second factor' % self.username)
                res = None
                for factor in rv['_embedded']['factors']:
                    if factor['factorType'] != 'token:software:totp':
                        continue
                    fid = factor['id']
                    state_token = rv['stateToken']
                    try:
                        res = self.doauth(fid, state_token)
                    except Exception as s:
                        log.error('Unexpected error with the Okta API: %s' % s)
                        return False

                    if 'status' in res and res['status'] == 'SUCCESS':
                        log.info('User %s is now authenticated with MFA via Okta API' % self.username)
                        return res['sessionToken']

                if 'errorCauses' in res:
                    msg = res['errorCauses'][0]['errorSummary']
                    log.debug('User %s MFA token authentication failed: %s' % (self.username, msg))
                return False
            log.info('User %s is not allowed to authenticate: %s' % (self.username, status))
            return False
            return


class OktaSamlAuth(OktaAPIAuth):

    def __init__(self, okta_url, application_type, application_id, username, password, passcode):
        self.application_type = application_type
        self.application_id = application_id
        OktaAPIAuth.__init__(self, okta_url, username, password, passcode)

    def saml(self, sessionToken):
        url = '{base}/app/{app}/{appid}/sso/saml'.format(base=self.okta_url, app=self.application_type, appid=self.application_id)
        resp = requests.get(url=url, params={'onetimetoken': sessionToken})

        if resp.status_code != 200:
            raise Exception('Received error code from server: %s' % resp.status_code)

        return resp.text

    def assertion(self, saml):
        assertion = ''
        soup = BeautifulSoup(saml, 'html.parser')
        for inputtag in soup.find_all('input'):
            if inputtag.get('name') == 'SAMLResponse':
                assertion = inputtag.get('value')

        decodedAssertion =  base64.b64decode(assertion)
        return str(decodedAssertion, "utf-8")

    def auth(self):
        token = super(OktaSamlAuth, self).auth()
        if not token:
            return False
        return self.assertion(self.saml(token))
